/*******************************************************************************
 * Copyright (c) 2009, 2013 Oracle. All rights reserved.
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0, which accompanies this distribution
 * and is available at https://www.eclipse.org/legal/epl-2.0/.
 * 
 * Contributors:
 *     Oracle - initial API and implementation
 ******************************************************************************/
package org.eclipse.jpt.jpa.eclipselink.core.internal.resource.java.binary;

import org.eclipse.jdt.core.IAnnotation;
import org.eclipse.jpt.common.core.internal.resource.java.binary.BinaryAnnotation;
import org.eclipse.jpt.common.core.resource.java.JavaResourceModel;
import org.eclipse.jpt.common.core.utility.TextRange;
import org.eclipse.jpt.jpa.eclipselink.core.resource.java.EclipseLink;
import org.eclipse.jpt.jpa.eclipselink.core.resource.java.ConversionValueAnnotation;

/**
 * <code>org.eclipse.persistence.annotations.ConversionValue</code>
 */
final class EclipseLinkBinaryConversionValueAnnotation
	extends BinaryAnnotation
	implements ConversionValueAnnotation
{
	private String dataValue;
	private String objectValue;


	EclipseLinkBinaryConversionValueAnnotation(JavaResourceModel parent, IAnnotation jdtAnnotation) {
		super(parent, jdtAnnotation);
		this.dataValue = this.buildDataValue();
		this.objectValue = this.buildObjectValue();
	}

	public String getAnnotationName() {
		return ANNOTATION_NAME;
	}

	@Override
	public void update() {
		super.update();
		this.setDataValue_(this.buildDataValue());
		this.setObjectValue_(this.buildObjectValue());
	}


	// ********** ConversionValueAnnotation implementation **********

	// ***** data value
	public String getDataValue() {
		return this.dataValue;
	}

	public void setDataValue(String dataValue) {
		throw new UnsupportedOperationException();
	}

	private void setDataValue_(String dataValue) {
		String old = this.dataValue;
		this.dataValue = dataValue;
		this.firePropertyChanged(DATA_VALUE_PROPERTY, old, dataValue);
	}

	private String buildDataValue() {
		return (String) this.getJdtMemberValue(EclipseLink.CONVERSION_VALUE__DATA_VALUE);
	}

	public TextRange getDataValueTextRange() {
		throw new UnsupportedOperationException();
	}

	// ***** object value
	public String getObjectValue() {
		return this.objectValue;
	}

	public void setObjectValue(String objectValue) {
		throw new UnsupportedOperationException();
	}

	private void setObjectValue_(String objectValue) {
		String old = this.objectValue;
		this.objectValue = objectValue;
		this.firePropertyChanged(OBJECT_VALUE_PROPERTY, old, objectValue);
	}

	private String buildObjectValue() {
		return (String) this.getJdtMemberValue(EclipseLink.CONVERSION_VALUE__OBJECT_VALUE);
	}

	public TextRange getObjectValueTextRange() {
		throw new UnsupportedOperationException();
	}
}
